(************************************************************
AgBlend Version 0.9

Author: Deepak Shenoy
        shenoy@agnisoft.com
Copyright (C) 2000 Agni Software Pvt. Ltd.
All Rights Reserved.
http://www.agnisoft.com

The AgBlend Component enables Alpha Blending for your forms. You can
drop it on a form, set the properties and then set Active = TRUE.
Blending is enabled only at Run time (no design time blending)

NEW- Works on Windows 9x and NT too. Thanks to John Bennett I now have
code for properly detecting the OS.

Properties
----------------------
BlendType:
  Alpha = Alpha blending ("fade" effect)
  ColorKey = makes one color transparent
  Opaque = nothing's transparent
TransparentColor :
  When BlendType = ColorKey, represents the
  color that will be transparent.
Opacity :
  When BlendType = Alpha, this gives the opacity
  (0-255). This is the extend to which the form
  "fades"

This is BETA software. All problems may be reported to
shenoy@agnisoft.com. Feel free to mail me any suggestions
or Updates.

Licensing:
This file is licensed to you as given in the readme.txt file accompanying this archive.
In short, you may redistribute this code as is, or with modifications - and use it
freely in any commercial or non-commercial applications.
But you may not license this code or any modifications to it under the GNU Public License (GPL).
(Agni Software does not endorse the GPL for components or code that it produces)

Credits (Other than me <g>):
Simon Lau - His article, at
http://www.cs.ucl.ac.uk/students/S.Lau/delphi004.htm
inspired me to make this component.
********************************************************)
unit AgBlend;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

const
  WS_EX_LAYERED = $80000;

  AC_SRC_OVER = $0;
  AC_SRC_ALPHA = $1;
  AC_SRC_NO_PREMULT_ALPHA = $1;
  AC1_SRC_NO_ALPHA = $2;
  AC_DST_NO_PREMULT_ALPHA = $10;
  AC_DST_NO_ALPHA = $20;
  LWA_COLORKEY = $1;
  LWA_ALPHA = $2;
  ULW_COLORKEY = $1;
  ULW_ALPHA =$2;
  ULW_OPAQUE = $4;

type
  TBlendType = ( ColorKey, Alpha, Opaque );
type
  TAgBlend = class(TComponent)
  private
    FTransparentColor: TColor;
    FOpacity: integer;
    FBlendType: TBlendType;
    FActive: boolean;
    FIsBlendingSupported : boolean;
    FhUser32 : THandle;
    procedure SetTransparentColor(const Value: TColor);
    procedure SetOpacity(const Value: integer);
    procedure SetBlendType(const Value: TBlendType);
    { Private declarations }
  protected
    { Protected declarations }
    function GetBlendValue : integer;
    procedure SetActive(const Value: boolean); virtual;
  public
    { Public declarations }
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;
    procedure Loaded; override;
    procedure Refresh;
    procedure Activate;
    procedure SetControlProperties( Control : TWinControl );

  published
    { Published declarations }
    property TransparentColor : TColor  read FTransparentColor write SetTransparentColor;
    property Opacity : integer read FOpacity write SetOpacity;
    property BlendType : TBlendType  read FBlendType write SetBlendType;
    property Active : boolean read FActive write SetActive;
  end;

procedure Register;

implementation

// 14.6.2001 changed to accomodate all windows versions
type TSetLayeredWindowAttributes = function
        (hwnd:HWND; crKey:Longint; bAlpha:byte; dwFlags:longint ):longint; stdcall;

var SetLayeredWindowAttributes : TSetLayeredWindowAttributes;
//function SetLayeredWindowAttributes(hwnd:HWND; crKey:Longint; bAlpha:byte; dwFlags:longint ):longint; stdcall; external 'user32.dll';

procedure Register;
begin
  RegisterComponents('Samples', [TAgBlend]);
end;

{ TAgBlend }

procedure TAgBlend.Activate;
begin
  FActive := TRUE;

  if (not (csDesigning in ComponentState)) and FIsBlendingSupported then
    SetLayeredWindowAttributes ((Owner as TForm).handle, FTransparentColor, FOpacity, GetBlendValue);
end;

constructor TAgBlend.Create(AOwner: TComponent);
begin
  inherited;

  FActive := FALSE;
  FOpacity := 255;

  FIsBlendingSupported := FALSE;
  FhUser32 := LoadLibrary('USER32.DLL');
  if FhUser32<>0 then
  begin
    @SetLayeredWindowAttributes := GetProcAddress(FhUser32, 'SetLayeredWindowAttributes');
    if @SetLayeredWindowAttributes <> nil then
      FIsBlendingSupported := TRUE;
  end;
end;

destructor TAgBlend.Destroy;
begin
  inherited;
  FreeLibrary(FhUser32);
end;

function TAgBlend.GetBlendValue: integer;
begin
  Result := 0;
  case FBlendType of
  ColorKey : Result :=LWA_COLORKEY;
  Alpha : Result := LWA_ALPHA ;
  Opaque : Result := ULW_OPAQUE;
  end;
end;

procedure TAgBlend.Loaded;
begin
  inherited;
  if not (csDesigning in ComponentState) and (FIsBlendingSupported) then
  begin
    SetControlProperties(Owner as TForm);

    if Active then
      Activate
    else
      SetLayeredWindowAttributes ((Owner as TForm).handle, 0, 255, ULW_ALPHA)
  end;
end;

procedure TAgBlend.Refresh;
begin
  Activate;
end;

procedure TAgBlend.SetActive(const Value: boolean);
begin
  FActive := Value;

  if FIsBlendingSupported then
    if not FActive then
    begin
        SetLayeredWindowAttributes ((Owner as TForm).handle, 0, 255, ULW_ALPHA)
    end
    else
      Activate;
end;

procedure TAgBlend.SetBlendType(const Value: TBlendType);
begin
  FBlendType := Value;
  if Active then
    Activate;
end;

procedure TAgBlend.SetControlProperties(Control: TWinControl);
var l : longint;
begin
  if (csDesigning in ComponentState) or (not FIsBlendingSupported) then Exit;

  l:=GetWindowLong(Control.Handle, GWL_EXSTYLE);
  l := l Or WS_EX_LAYERED;
  SetWindowLong (Control.handle, GWL_EXSTYLE, l);
end;

procedure TAgBlend.SetOpacity(const Value: integer);
begin
  if Value < 0 then FOpacity := 0
  else if Value > 255 then FOpacity := 255
  else FOpacity := Value;
  if Active then
    Activate;
end;

procedure TAgBlend.SetTransparentColor(const Value: TColor);
begin
  FTransparentColor := Value;
  if Active then
    Activate;
end;

end.
